/*-
 * Copyright (c) 2011 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/uuid.h>

#include <inttypes.h>
#include <regex.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include "virtdisk.h"

int
main(int argc, char **argv)
{
	const char	*virtdisktype;
	const char	*type;
	uint64_t	 size;
	uint32_t	 blocksize;
	char		 op;
	int		 ex;
	int		 i;

	size = VIRTDISK_MB(64);
	blocksize = VIRTDISK_MB(2);
	op = 0x0;
	type = "vmdk";
	virtdisktype = "fixed";
	while ((i = getopt(argc, argv, "B:T:cls:t:")) != -1) {
		switch(i) {
		case 'B':
			blocksize = atoi(optarg);
			break;
		case 'T':
			virtdisktype = optarg;
			break;
		case 'c':
		case 'l':
			op = (char)i;
			break;
		case 's':
			size = atoi(optarg);
			break;
		case 't':
			type = optarg;
			break;
		default:
			break;
		}
	}
	ex = EXIT_SUCCESS;
	switch(op) {
	case 'c':
		/* create */
		if (!virtdisk_create(type, argv[optind], virtdisktype, size, blocksize)) {
			ex = EXIT_FAILURE;
		}
		break;
	case 'l':
		if (!virtdisk_list(argv[optind])) {
			ex = EXIT_FAILURE;
		}
		break;
	default:
		break;
	}
	exit(ex);
}
