/*-
 * Copyright (c) 2010 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef VHD_H_
#define VHD_H_	20101221

#include <inttypes.h>
#include <stdio.h>

enum {
	VHD_FEATURES_TEMPORARY	= 0x00000001,
	VHD_FEATURES_RESERVED	= 0x00000002,

	VHD_FORMAT_VERSION	= 0x00010000,
	VHD_FIXED_OFFSET	= 0xffffffff,

	VIRTUAL_SERVER_2004	= 0x00010000,
	VIRTUAL_PC_2004		= 0x00050000,
	NETBSD_VHD		= 0x00100000,

	VHD_TYPE_NONE		= 0,
	VHD_TYPE_RESERVED	= 1,
	VHD_TYPE_FIXED		= 2,
	VHD_TYPE_DYNAMIC	= 3,
	VHD_TYPE_DIFFERENCING	= 4,
	VHD_TYPE_RESERVED5	= 5,
	VHD_TYPE_RESERVED6	= 6,
};

#define VHD_FORMAT_MAJOR(x)	(((x) >> 16) & 0xffff)
#define VHD_FORMAT_MINOR(x)	((x) & 0xffff)

#define SECS_TO_VHD_TIME(x)	((x) + 946742400)
#define VHD_TIME_TO_SECS(x)	((x) - 946742400)

#define VHD_HOST_OS_WINDOWS	"Wi2k"
#define VHD_HOST_OS_MAC		"Mac "
#define VHD_HOST_OS_NETBSD	"NBSD"
#define VHD_HEADER_COOKIE	"cxsparse"

#define VDISK_GEOM(cyl,hd,sec)	(((cyl) << 16) | ((hd) << 8) | (sec))
#define VDISK_CYLS(x)		(((x) >> 16) & 0xffff)
#define VDISK_HEADS(x)		(((x) >> 8) & 0xff)
#define VDISK_SECTORS(x)		((x) & 0xff)

typedef struct vhd_footer_t {
	uint8_t		cookie[8];
	uint32_t	features;
	uint32_t	version;
	uint64_t	data_offset;
	uint32_t	timestamp;
	uint8_t		maker_app[4];
	uint32_t	maker_version;
	uint8_t		maker_host_os[4];
	uint64_t	origsize;
	uint64_t	size;
	uint32_t	geom;
	uint32_t	disktype;
	uint32_t	cksum;
	uint8_t		uuid[16];
	uint8_t		saved_state;
	uint8_t		reserved[427];
} vhd_footer_t;

typedef struct locator_t {
	uint32_t	code;
	uint32_t	data_space;
	uint32_t	data_length;
	uint32_t	reserved;
	uint64_t	offset;
} locator_t;

typedef struct vhd_header_t {
	uint8_t		cookie[8];
	uint64_t	data_offset;
	uint64_t	table_offset;
	uint32_t	version;
	uint32_t	maxent;
	uint32_t	blocksize;
	uint32_t	cksum;
	uint8_t		parent[16];
	uint32_t	parent_timestamp;
	uint32_t	reserved;
	uint8_t		parent_name[512];
	locator_t	locators[8];
	uint8_t		reserved2[256];
} vhd_header_t;

int vhd_attach(const char *, FILE *, vhd_header_t *, vhd_footer_t *, uint32_t **);
int vhd_create(uint8_t *, const char *, uint64_t, uint32_t, uint32_t);
int vhd_list(const char *);
uint64_t vhd_size(const char *, uint64_t, uint32_t);
int vhd_read_footer(FILE *, vhd_footer_t *);

#endif
