/*-
 * Copyright (c) 2010 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef VMDK_H_
#define VMDK_H_	20101223

#include <inttypes.h>
#include <stdio.h>

#define VMDK_KB(x)	((x) * 1024)
#define VMDK_MB(x)	((x) * 1024 * 1024)

enum {
	VMDK_MAGIC		= 0x564d444b,	/* VMDK */

	VMDK_SECTOR_SIZE	= 512,

	VMDK_DEFAULT_GRAIN_SIZE	= (VMDK_KB(64) / VMDK_SECTOR_SIZE),

	VMDK_DEFAULT_EXTENT_SIZE = 0x1000000,

	VALID_NEWLINE_TEST	= 1,
	REDUNDANT_GRAIN_TABLE	= (1 << 1),
	COMPRESSED_GRAIN_TABLE	= (1 << 16),
	LBA_ADDRESSING		= (1 << 17),
	COMPRESSION_DEFLATE	= 1
};

typedef struct SparseExtentHeader { 
	uint32_t	magicNumber; 
	uint32_t	version; 
	uint32_t	flags; 
	uint64_t	capacity; 
	uint64_t	grainSize; 
	uint64_t	descriptorOffset; 
	uint64_t	descriptorSize; 
	uint32_t	numGTEsPerGT; 
	uint64_t	rgdOffset; 
	uint64_t	gdOffset; 
	uint64_t	overHead; 
	uint8_t		uncleanShutdown; 
	char		singleEndLineChar; 
	char		nonEndLineChar; 
	char		doubleEndLineChar1; 
	char		doubleEndLineChar2; 
	uint16_t	compressAlgorithm;
	uint8_t		pad[433]; 
} SparseExtentHeader;

int vmdk_create(uint8_t *, const char *, uint64_t, uint32_t, uint32_t);
uint64_t vmdk_size(const char *, uint64_t, uint32_t);
int vmdk_list(const char *);
int vmdk_read_header(FILE *, SparseExtentHeader *);

#endif
