/*	$NetBSD: vnconfig.c,v 1.38 2009/12/06 16:35:16 dsl Exp $	*/

/*-
 * Copyright (c) 1997 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Jason R. Thorpe.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Copyright (c) 1990, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * the Systems Programming Group of the University of Utah Computer
 * Science Department.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * from: Utah $Hdr: vnconfig.c 1.1 93/12/15$
 *
 *	@(#)vnconfig.c	8.1 (Berkeley) 12/15/93
 */

/*
 * Copyright (c) 1993 University of Utah.
 *
 * This code is derived from software contributed to Berkeley by
 * the Systems Programming Group of the University of Utah Computer
 * Science Department.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * from: Utah $Hdr: vnconfig.c 1.1 93/12/15$
 *
 *	@(#)vnconfig.c	8.1 (Berkeley) 12/15/93
 */

#include <sys/param.h>
#include <sys/ioctl.h>
#include <sys/mount.h>
#include <sys/buf.h>
#include <sys/disklabel.h>
#include <sys/disk.h>

#include <dev/vndvar.h>

#include <disktab.h>
#include <err.h>
#include <errno.h>
#include <fcntl.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <util.h>

#include "vndconfig.h"

static void
usage(void)
{

	(void)fprintf(stderr, "%s%s",
	    "usage: vndconfig [-cmrvz] [-f disktab] [-t typename] vnode_disk"
		" regular-file [geomspec]\n",
	    "       vndconfig -u [-Fv] vnode_disk\n"
	    "       vndconfig -l [vnode_disk]\n");
	exit(EXIT_SUCCESS);
}

int
main(int argc, char **argv)
{
	vnd_args_t	args;
	int ch, ok, action = VND_CONFIG;

	(void) memset(&args, 0x0, sizeof(args));
	while ((ch = getopt(argc, argv, "Fcf:lrt:uvz")) != -1) {
		switch (ch) {
		case 'F':
			args.force = 1;
			break;
		case 'c':
			action = VND_CONFIG;
			break;
		case 'f':
			if (setdisktab(optarg) == -1) {
				usage();
			}
			break;
		case 'l':
			action = VND_GET;
			break;
		case 'r':
			args.readonly = 1;
			break;
		case 't':
			if (strcmp(optarg, "vhd") == 0) {
				args.vhd = 1;
			} else {
				args.tabname = optarg;
			}
			break;
		case 'u':
			action = VND_UNCONFIG;
			break;
		case 'v':
			args.verbose += 1;
			break;
		case 'z':
			args.compressed = 1;
			args.readonly = 1;
			break;
		default:
		case '?':
			usage();
			/* NOTREACHED */
		}
	}
	argc -= optind;
	argv += optind;
	if (args.vhd && args.compressed) {
		usage();
		/* NOTREACHED */
	}
	if (action == VND_CONFIG) {
		if ((argc < 2 || argc > 3) ||
		    (argc == 3 && args.tabname != NULL)) {
			usage();
		}
		ok = vndconfig(&args, argv[0], argv[1], (argc == 3) ? argv[2] : NULL,
		    action);
	} else if (action == VND_UNCONFIG) {
		if (argc != 1 || args.tabname != NULL) {
			usage();
		}
		ok = vndconfig(&args, argv[0], NULL, NULL, action);
	} else { /* VND_GET */
		const char *vn;
		char path[64];
		struct vnd_user vnu;
		int v, n;

		ok = 1;
		if (argc != 0 && argc != 1) {
			usage();
		}
		vn = (argc) ? argv[0] : "vnd0";

		v = opendisk(vn, O_RDONLY, path, sizeof(path), 0);
		if (v == -1) {
			err(EXIT_FAILURE, "open: %s", vn);
		}
		for (n = 0; ; n++) {
			vnu.vnu_unit = (argc) ? -1 : n;
			if (ioctl(v, VNDIOCGET, &vnu) != 0) {
				ok = 0;
				if (errno == ENXIO) {
					break;
				}
				err(EXIT_FAILURE, "VNDIOCGET");
			}

			if (vnu.vnu_ino == 0) {
				printf("vnd%d: not in use\n",
				    vnu.vnu_unit);
			} else {
				char *dev;
				struct statvfs *mnt = NULL;
				int i, nmount;

				nmount = 0;	/* XXXGCC -Wuninitialized */

				printf("vnd%d: ", vnu.vnu_unit);

				dev = devname(vnu.vnu_dev, S_IFBLK);
				if (dev != NULL) {
					nmount = getmntinfo(&mnt, MNT_NOWAIT);
				} else {
					mnt = NULL;
				}
				if (mnt != NULL) {
					for (i = 0; i < nmount; i++) {
						if (strncmp(
						    mnt[i].f_mntfromname,
						    "/dev/", 5) == 0 &&
						    strcmp(
						    mnt[i].f_mntfromname + 5,
						    dev) == 0) {
							break;
						 }
					}
					if (i < nmount) {
						printf("%s (%s) ",
						    mnt[i].f_mntonname,
						    mnt[i].f_mntfromname);
					} else {
						printf("%s ", dev);
					}
				} else if (dev != NULL) {
					printf("%s ", dev);
				} else {
					printf("dev %llu,%llu ",
					    (unsigned long long)major(vnu.vnu_dev),
					    (unsigned long long)minor(vnu.vnu_dev));
				}
				printf("inode %llu\n",
				    (unsigned long long)vnu.vnu_ino);
			}
			if (argc) {
				break;
			}
		}
		close(v);
	}
	exit((ok) ? EXIT_SUCCESS : EXIT_FAILURE);
}
